#ifndef K3DSDK_MESH_SOURCE_H
#define K3DSDK_MESH_SOURCE_H

#include "imesh_source.h"
#include "mesh.h"
#include "property_collection.h"
#include "selection.h"

namespace k3d
{

template<typename base_t>
class mesh_source :
	public base_t,
	public imesh_source
{
public:
	mesh_source(idocument& Document) :
		base_t(Document),
		m_output_mesh(init_name("output_mesh") + init_description("Output mesh [mesh]") + init_document(Document))
	{
		register_property(m_output_mesh);
		
		Document.selection().deselect_all_signal().connect(SigC::slot(*this, &mesh_source::on_deselect_all));
	}

	iproperty& mesh_source_output()
	{
		return m_output_mesh;
	}

protected:
	k3d_read_only_data_property(mesh*, immutable_name, change_signal, no_undo, demand_storage, no_constraint) m_output_mesh;

private:
	void on_deselect_all()
	{
		if(m_output_mesh.empty())
			return;
		k3d::deselect(document(), deep_selection(document().dag(), make_selection(*m_output_mesh.value())));
	}
};

} // namespace k3d

#endif // !K3DSDK_MESH_SOURCE_H

