#ifndef MODULE_BITMAP_BITMAP_COMPOSITE_H
#define MODULE_BITMAP_BITMAP_COMPOSITE_H

#include <k3dsdk/ibitmap_sink.h>
#include <k3dsdk/ibitmap_source.h>
#include <k3dsdk/property_collection.h>
#include <k3dsdk/bitmap.h>

namespace libk3dbitmap
{

template<typename base_t>
class bitmap_composite :
	public base_t,
	public k3d::ibitmap_source,
	public k3d::ibitmap_sink
{
public:
	bitmap_composite(k3d::idocument& Document) :
		base_t(Document),
		m_bitmap_a(k3d::init_name("input_a") + k3d::init_description("A input bitmap [bitmap]") + k3d::init_value<k3d::bitmap*>(0) + k3d::init_document(Document)),
		m_bitmap_b(k3d::init_name("input_b") + k3d::init_description("B input bitmap [bitmap]") + k3d::init_value<k3d::bitmap*>(0) + k3d::init_document(Document)),
		m_bitmap_o(k3d::init_name("output") + k3d::init_description("Output bitmap [bitmap]") + k3d::init_document(Document))
	{
		register_property(m_bitmap_a);
		register_property(m_bitmap_b);
		register_property(m_bitmap_o);
	}

	k3d::iproperty& bitmap_source_output()
	{
		return m_bitmap_o;
	}
	
	k3d::iproperty& bitmap_sink_input()
	{
		return m_bitmap_a;
	}

protected:
	k3d_data_property(k3d::bitmap*, k3d::immutable_name, k3d::change_signal, k3d::no_undo, k3d::local_storage, k3d::no_constraint) m_bitmap_a;
	k3d_data_property(k3d::bitmap*, k3d::immutable_name, k3d::change_signal, k3d::no_undo, k3d::local_storage, k3d::no_constraint) m_bitmap_b;
	k3d_read_only_data_property(k3d::bitmap*, k3d::immutable_name, k3d::change_signal, k3d::no_undo, k3d::demand_storage, k3d::no_constraint) m_bitmap_o;
};

} // namespace libk3dbitmap

#endif // !MODULE_BITMAP_BITMAP_COMPOSITE_H

